/*  Copyright 2010-2011 Guillaume Duhamel

    This file is part of Gophrier.

    Gophrier is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gophrier is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gophrier; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/

#include "dir.h"
#include "sock.h"
#include "request.h"
#include "conf.h"

#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

static void dir_push(Request * request, char type, const char * user_name, const char * selector, const char * host, unsigned int port);
static void dir_push_file(Request * request, const char * user_name, const char * selector, const char * host, unsigned int port);
static void dir_close(Request * request);

void dir_parse(Request * request)
{
    DIR * d;
    struct dirent * current;
    struct stat info;

    d = opendir(request->fullpath);
    while(NULL != (current = readdir(d)))
    {
        char fullname[1024], absname[1024];
        if (current->d_name[0] == '.') continue;
    
        sprintf(absname, "%s/%s", request->path, current->d_name);
        sprintf(fullname, "%s%s", conf_get_root_dir(), absname);
        stat(fullname, &info);
        if (S_ISDIR(info.st_mode))
            dir_push(request, '1', current->d_name, absname, request->sock->name, conf_get_port());
        else if (S_ISREG(info.st_mode))
            dir_push_file(request, current->d_name, absname, request->sock->name, conf_get_port());
        else
            dir_push(request, 'i', current->d_name, "fake", "(NULL)", 0);
    }
    closedir(d);
    
    dir_close(request);
}

static void dir_push(Request * request, char type, const char * name, const char * path, const char * host, unsigned int port)
{
    char buffer[1024];
    sprintf(buffer, "%c%s\t%s\t%s\t%d\r\n", type, name, path, host, port);
    request_send(request, buffer, strlen(buffer));
}

static void dir_push_file(Request * request, const char * name, const char * path, const char * host, unsigned int port)
{
    char * ext;

    ext = strrchr(path, '.');
    if (ext)
    {
        if (! strcmp(ext, ".html"))
        {
            dir_push(request, 'h', name, path, host, port);
            return;
        }
        else if ((! strcmp(ext, ".png")) || (! strcmp(ext, ".jpg")))
        {
            dir_push(request, 'I', name, path, host, port);
            return;
        }
    }

    dir_push(request, 'i', name, "fake", "(NULL)", 0);
}

static void dir_close(Request * request)
{
    request_send(request, ".\r\n", 3);
    close(request->sock->fd);
}
