/*  Copyright 2010-2011 Guillaume Duhamel

    This file is part of Gophrier.

    Gophrier is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gophrier is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gophrier; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/

#include <stdlib.h>
#include <netdb.h>

#include "sock.h"
#include "conf.h"

Socket * socket_new(int fd)
{
    Socket * sock;

    sock = (Socket *) malloc(sizeof(Socket));
    sock->fd = fd;
    sock->onRead = NULL;

    {
        struct sockaddr * myaddr;
        socklen_t size;
        char host[1024];

        if (conf_use_ipv6())
            size = sizeof(struct sockaddr_in6);
        else
            size = sizeof(struct sockaddr_in);
        myaddr = malloc(size);
    
        getsockname(sock->fd, myaddr, &size);
        getnameinfo(myaddr, size, host, 1024, NULL, 0, NI_NUMERICHOST);
        sock->name = strdup(host);

        getpeername(sock->fd, myaddr, &size);
        getnameinfo(myaddr, size, host, 1024, NULL, 0, NI_NUMERICHOST);
        sock->peer = strdup(host);

        free(myaddr);
    }

    return sock;
}

void socket_free(Socket * sock)
{
    free(sock->name);
    free(sock->peer);
    free(sock);
}
